@@ -2,7 +2,7 @@
 /**
  * sieve-php.lib.php
  *
- * $Id: managesieve.lib.php,v 1.7 2006-06-14 09:13:02 avel Exp $ 
+ * $Id: managesieve.lib.php 1030 2009-05-25 08:19:28Z avel $ 
  *
  * Copyright 2001-2003 Dan Ellis <danellis@rushmore.com>
  *
@@ -16,7 +16,7 @@
  *
  * See CHANGES for updates since last release
  *
- * @author Dan Ellis
+ * @author Dan Ellis, Alexandros Vellis
  * @package sieve-php
  * @copyright Copyright 2002-2003, Dan Ellis, All Rights Reserved.  
  * @version 0.1.0
@@ -25,7 +25,7 @@
 /**
  * Constants
  */
-define ("F_NO", 0);		
+define ("F_NO", 0);        
 define ("F_OK", 1);
 define ("F_DATA", 2);
 define ("F_HEAD", 3);
@@ -82,11 +82,22 @@ class sieve {
    * a comma seperated list of allowed auth types, in order of preference
    */
   var $auth_types;
+
+  /** 
+   * options
+   */
+  var $broken_tls;
+
   /**
    * type of authentication attempted
    */
   var $auth_in_use;
   
+  /**
+   * @var boolean Force disabling of STARTTLS for clients that do not want/need 
+   * it. */
+  var $disabletls = false;
+
   var $line;
   var $fp;
   var $retval;
@@ -151,7 +162,7 @@ class sieve {
                 $this->error_raw[]=substr($this->line, 0, strlen($this->line) -2);    //we want to be nice and strip crlf's
                 $this->err_recv += strlen($this->line);
             } /* end while */
-            $this->line = fgets($this->fp, 1024);	//we need to grab the last crlf, i think.  this may be a bug...
+            $this->line = fgets($this->fp, 1024);    //we need to grab the last crlf, i think.  this may be a bug...
             $this->error=EC_UNKNOWN;
       
         } /* end if */
@@ -202,7 +213,7 @@ class sieve {
         while(substr($this->line,0,2) != "OK" and substr($this->line,0,2) != "NO"){
             $this->found_script=true;
             list($this->ltoken, $this->rtoken) = explode(" ", $this->line." ",2);
-		//hmmm, a bug in php, if there is no space on explode line, a warning is generated...
+        //hmmm, a bug in php, if there is no space on explode line, a warning is generated...
            
             if(strcmp(rtrim($this->rtoken), "ACTIVE")==0){
                 $this->response["ACTIVE"] = substr(rtrim($this->ltoken),1,-1);  
@@ -216,54 +227,54 @@ class sieve {
         
     } /* end elseif */
     elseif(strstr($this->token[1], '(REFERRAL "' ) ){
-    	/* process a referral, retry the lastcmd, return the results.  this is 
-    	   sort of messy, really I should probably try to use parse_for_quotes
-    	   but the problem is I still have the ( )'s to deal with.  This is 
-    	   atleast true for timsieved as it sits in 2.1.16, if someone has a 
-    	   BYE (REFERRAL ...) example for later timsieved please forward it to 
-    	   me and I'll code it in proper-like! - mloftis@wgops.com */
-    	$this->reftok = split(" ", $this->token[1], 3);
-    	$this->refsv = substr($this->reftok[1], 0, -2);
-    	$this->refsv = substr($this->refsv, 1);
+        /* process a referral, retry the lastcmd, return the results.  this is 
+           sort of messy, really I should probably try to use parse_for_quotes
+           but the problem is I still have the ( )'s to deal with.  This is 
+           atleast true for timsieved as it sits in 2.1.16, if someone has a 
+           BYE (REFERRAL ...) example for later timsieved please forward it to 
+           me and I'll code it in proper-like! - mloftis@wgops.com */
+        $this->reftok = split(" ", $this->token[1], 3);
+        $this->refsv = substr($this->reftok[1], 0, -2);
+        $this->refsv = substr($this->refsv, 1);
 
         /* TODO - perform more testing */
         if(strstr($this->capabilities['implementation'], 'v2.1')) {
             /* Cyrus 2.1 - Style referrals */
-        	$this->host = $this->refsv;
+            $this->host = $this->refsv;
         } else {
             /* Cyrus 2.2 - Style referrals */
             $tmp = array_reverse( explode( '/', $this->refsv ) );
             $this->host = $tmp[0];
         }
-    	$this->loggedin = false;
-    	/* flush buffers or anything?  probably not, and the remote has already closed it's
-    	   end by now!  */
-    	fclose($this->fp);
-    	
-    	if( sieve::sieve_login() ) {
-    		fputs($this->fp, $this->lastcmd);
-    		return sieve::get_response();
-    	} /* end good case happy ending */
-    	else{
-    		/* what to do?  login failed, should we punt and die? or log back into the referrer?
-    		   i'm electing to retn EC_UNKNOWN for now and set the error string. */
-    		$this->loggedin = false;
-    		fclose($this->fp);
-    		$this->error = EC_UNKNOWN;
-    		$this->error_raw = 'UNABLE TO FOLLOW REFERRAL - ' . $this->line;
-    		return false;
-    	} /* end else of the unhappy ending */
-    	
-    	/* should never make it here! */
-    	
+        $this->loggedin = false;
+        /* flush buffers or anything?  probably not, and the remote has already closed it's
+           end by now!  */
+        fclose($this->fp);
+        
+        if( sieve::sieve_login() ) {
+            fputs($this->fp, $this->lastcmd);
+            return sieve::get_response();
+        } /* end good case happy ending */
+        else{
+            /* what to do?  login failed, should we punt and die? or log back into the referrer?
+               i'm electing to retn EC_UNKNOWN for now and set the error string. */
+            $this->loggedin = false;
+            fclose($this->fp);
+            $this->error = EC_UNKNOWN;
+            $this->error_raw = 'UNABLE TO FOLLOW REFERRAL - ' . $this->line;
+            return false;
+        } /* end else of the unhappy ending */
+        
+        /* should never make it here! */
+        
     } /* end elseif */
     else{
             $this->error = EC_UNKNOWN;
             $this->error_raw = $this->line;
-	    print '<b><i>UNKNOWN ERROR (Please report this line to <a
-	    href="mailto:sieve-php-devel@lists.sourceforge.net">sieve-php-devel
-	    Mailing List</a> to include in future releases):
-	    '.$this->line.'</i></b><br>';
+        print '<b><i>UNKNOWN ERROR (Please report this line to <a
+        href="mailto:sieve-php-devel@lists.sourceforge.net">sieve-php-devel
+        Mailing List</a> to include in future releases):
+        '.$this->line.'</i></b><br>';
 
             return false;
     } /* end else */   
@@ -282,13 +293,13 @@ class sieve {
    * @param $port string Numeric port to connect to. SIEVE daemons usually
    * listen to port 2000.
    *
-   * @param $user string is a super-user or proxy-user that has ACL rights to
-   * login on behalf of the $auth.
+   * @param $user string is the  user identity for which the SIEVE scripts
+   * will be managed (also know as authcid).
    *
    * @param $pass string password to use for authentication
    *
-   * @param $auth string is the authorized user identity for which the SIEVE
-   * scripts will be managed.
+   * @param $auth string is a super-user or proxy-user that has ACL rights to
+   * login on behalf of the $auth (also know as authzid).
    *
    * @param $auth_types string a string containing all the allowed
    * authentication types allowed in order of preference, seperated by spaces.
@@ -306,11 +317,14 @@ class sieve {
     $this->port=$port;
     $this->user=$user;
     $this->pass=$pass;
-    if(!strcmp($auth, ""))		/* If there is no auth user, we deem the user itself to be the auth'd user */
+    if(!strcmp($auth, ""))        /* If there is no auth user, we deem the user itself to be the auth'd user */
         $this->auth = $this->user;
     else
         $this->auth = $auth;
-    $this->auth_types=$auth_types;	/* Allowed authentication types */
+    $this->auth_types=$auth_types;    /* Allowed authentication types */
+    
+    $this->broken_tls = false;
+
     $this->fp=0;
     $this->line="";
     $this->retval="";
@@ -369,8 +383,8 @@ class sieve {
 
       switch (substr($string, 0,2)){
           case "NO":
-              return F_NO;		//there should be some function to extract the error code from this line
-					//NO ("quota") "You are oly allowed x number of scripts"
+              return F_NO;        //there should be some function to extract the error code from this line
+                    //NO ("quota") "You are oly allowed x number of scripts"
               break;
           case "OK":
               return F_OK;
@@ -412,7 +426,6 @@ class sieve {
    * @return boolean
    */
   function sieve_login() {
-
     $this->fp=@fsockopen($this->host,$this->port, $errno, $errstr);
     if($this->fp == false) {
         $this->error = $errno. ' '.$errstr;
@@ -450,10 +463,10 @@ class sieve {
                   $this->capabilites[$this->cap_type][$this->modules]=true;
           }    
           elseif(strcmp($this->item[0], "STARTTLS") == 0) {
-	          $this->capabilities['starttls'] = true;
-	  
+              $this->capabilities['starttls'] = true;
+      
           }
-	  else{ 
+      else{ 
               $this->capabilities["unknown"][]=$this->line;
           }    
       $this->line=fgets($this->fp,1024);
@@ -484,9 +497,13 @@ class sieve {
         } /* end if */
         elseif(is_string($this->modules))
             $this->capabilites[$this->cap_type][$this->module]=true;
+
+    // set broken_tls. Older cyrus servers do not respond with 
+    // capabilities after STARTTLS
+    $broken_tls = true;
     }
 
-    if(sieve::status($this->line) == F_NO){		//here we should do some returning of error codes?
+    if(sieve::status($this->line) == F_NO){        //here we should do some returning of error codes?
         $this->error=EC_UNKNOWN;
         $this->error_raw = "Server not allowing connections.";
         return false;
@@ -495,7 +512,8 @@ class sieve {
     /* decision login to decide what type of authentication to use... */
 
     /* If we allow STARTTLS, use it */ 
-    if($this->capabilities['starttls'] === true && function_exists('stream_socket_enable_crypto') === true) {
+    if(isset($this->capabilities['starttls']) && $this->capabilities['starttls'] === true &&
+      function_exists('stream_socket_enable_crypto') === true && !$this->disabletls ) {
         fputs($this->fp,"STARTTLS\r\n");
         $starttls_response = $this->line=fgets($this->fp,1024);
         if(stream_socket_enable_crypto($this->fp, true, STREAM_CRYPTO_METHOD_TLS_CLIENT) == false) {
@@ -504,8 +522,13 @@ class sieve {
             return false;
         } else {
             $this->loggedin = true;
-            // RFC says that we need to ask for the capabilities again
-            $this->sieve_get_capability();
+            // RFC says that we get an unsolicited capability response after TLS negotiation. Older Cyrus
+            // did not do this. If the server has old/broken TLS we need to send a CAPABILITY command,
+            // otherwise we just parse the unsolicited capability response. 
+            if ( $this->broken_tls )
+                $this->sieve_get_capability();
+            else
+                $this->sieve_read_capability_response();
             $this->loggedin = false;
         }
     }
@@ -596,7 +619,7 @@ class sieve {
     if($this->loggedin==false)
         return false;
     
-		$this->lastcmd = "SETACTIVE \"$scriptname\"\r\n";
+        $this->lastcmd = "SETACTIVE \"$scriptname\"\r\n";
     fputs($this->fp, $this->lastcmd);   
     return sieve::get_response();
 
@@ -633,8 +656,13 @@ class sieve {
   function sieve_deletescript($scriptname)   {
     if($this->loggedin==false)
         return false;
-        
-		$this->lastcmd = "DELETESCRIPT \"$scriptname\"\r\n";
+
+    // If there is an active script, "inactivate" it first.
+    $this->sieve_listscripts(); 
+    if ($this->response['ACTIVE'] === $scriptname) 
+        $this->sieve_setactivescript(""); 
+
+    $this->lastcmd = "DELETESCRIPT \"$scriptname\"\r\n"; 
     fputs($this->fp, $this->lastcmd);    
 
     return sieve::get_response();
@@ -651,13 +679,13 @@ class sieve {
    * @return boolean
    */
   function sieve_listscripts() { 
-  	 $this->lastcmd = "LISTSCRIPTS\r\n";
+       $this->lastcmd = "LISTSCRIPTS\r\n";
      fputs($this->fp, $this->lastcmd); 
-     sieve::get_response();		//should always return true, even if there are no scripts...
+     sieve::get_response();        //should always return true, even if there are no scripts...
      if(isset($this->found_script) and $this->found_script)
          return true;
      else{
-         $this->error=EC_NOSCRIPTS;	//sieve::getresponse has no way of telling wether a script was found...
+         $this->error=EC_NOSCRIPTS;    //sieve::getresponse has no way of telling wether a script was found...
          $this->error_raw="No scripts found for this account.";
          return false;
      }
@@ -677,7 +705,7 @@ class sieve {
           $this->error = EC_NOT_LOGGED_IN;
           return false;
       }
-      elseif(feof($this->fp)){			
+      elseif(feof($this->fp)){            
           $this->error = EC_NOT_LOGGED_IN;
           return false;
       }
@@ -697,13 +725,13 @@ class sieve {
     switch ($this->auth_in_use) {
 
         case "PLAIN":
-            $auth=base64_encode("$this->auth\0$this->user\0$this->pass");
+            $auth=base64_encode($this->user."\0".$this->auth."\0".$this->pass);
    
-            $this->len=strlen($auth);			
+            $this->len=strlen($auth);            
             fputs($this->fp, 'AUTHENTICATE "PLAIN" {' . $this->len . '+}' . "\r\n");
             fputs($this->fp, "$auth\r\n");
 
-            $this->line=fgets($this->fp,1024);		
+            $this->line=fgets($this->fp,1024);        
             while(sieve::status($this->line) == F_DATA)
                $this->line=fgets($this->fp,1024);
 
@@ -711,47 +739,47 @@ class sieve {
                return false;
              $this->loggedin=true;
                return true;    
-	    break;
-	
+        break;
+    
         case "DIGEST-MD5":
-	     // SASL DIGEST-MD5 support works with timsieved 1.1.0
-	     // follows rfc2831 for generating the $response to $challenge
-	     fputs($this->fp, "AUTHENTICATE \"DIGEST-MD5\"\r\n");
-	     // $clen is length of server challenge, we ignore it. 
-	     $clen = fgets($this->fp, 1024);
-	     // read for 2048, rfc2831 max length allowed
-	     $challenge = fgets($this->fp, 2048);
-	     // vars used when building $response_value and $response
-	     $cnonce = base64_encode(bin2hex(hmac_md5(microtime())));
-	     $ncount = "00000001";
-	     $qop_value = "auth"; 
-	     $digest_uri_value = "sieve/$this->host";
-	     // decode the challenge string
-	     $result = decode_challenge($challenge);
-	     // verify server supports qop=auth 
-	     $qop = explode(",",$result['qop']);
-	     if (!in_array($qop_value, $qop)) {
-	        // rfc2831: client MUST fail if no qop methods supported
-	        return false;
-	     }
-	     // build the $response_value
-	     $string_a1 = utf8_encode($this->user).":";
-	     $string_a1 .= utf8_encode($result['realm']).":";
-	     $string_a1 .= utf8_encode($this->pass);
-	     $string_a1 = hmac_md5($string_a1);
-	     $A1 = $string_a1.":".$result['nonce'].":".$cnonce.":".utf8_encode($this->auth);
-	     $A1 = bin2hex(hmac_md5($A1));
-	     $A2 = bin2hex(hmac_md5("AUTHENTICATE:$digest_uri_value"));
-	     $string_response = $result['nonce'].":".$ncount.":".$cnonce.":".$qop_value;
-	     $response_value = bin2hex(hmac_md5($A1.":".$string_response.":".$A2));
-	     // build the challenge $response
-	     $reply = "charset=utf-8,username=\"".$this->user."\",realm=\"".$result['realm']."\",";
-	     $reply .= "nonce=\"".$result['nonce']."\",nc=$ncount,cnonce=\"$cnonce\",";
-	     $reply .= "digest-uri=\"$digest_uri_value\",response=$response_value,";
-	     $reply .= "qop=$qop_value,authzid=\"".utf8_encode($this->auth)."\"";
-	     $response = base64_encode($reply);
-	     fputs($this->fp, "\"$response\"\r\n");
- 	
+         // SASL DIGEST-MD5 support works with timsieved 1.1.0
+         // follows rfc2831 for generating the $response to $challenge
+         fputs($this->fp, "AUTHENTICATE \"DIGEST-MD5\"\r\n");
+         // $clen is length of server challenge, we ignore it. 
+         $clen = fgets($this->fp, 1024);
+         // read for 2048, rfc2831 max length allowed
+         $challenge = fgets($this->fp, 2048);
+         // vars used when building $response_value and $response
+         $cnonce = base64_encode(bin2hex(hmac_md5(microtime())));
+         $ncount = "00000001";
+         $qop_value = "auth"; 
+         $digest_uri_value = "sieve/$this->host";
+         // decode the challenge string
+         $result = decode_challenge($challenge);
+         // verify server supports qop=auth 
+         $qop = explode(",",$result['qop']);
+         if (!in_array($qop_value, $qop)) {
+            // rfc2831: client MUST fail if no qop methods supported
+            return false;
+         }
+         // build the $response_value
+         $string_a1 = utf8_encode($this->user).":";
+         $string_a1 .= utf8_encode($result['realm']).":";
+         $string_a1 .= utf8_encode($this->pass);
+         $string_a1 = hmac_md5($string_a1);
+         $A1 = $string_a1.":".$result['nonce'].":".$cnonce.":".utf8_encode($this->auth);
+         $A1 = bin2hex(hmac_md5($A1));
+         $A2 = bin2hex(hmac_md5("AUTHENTICATE:$digest_uri_value"));
+         $string_response = $result['nonce'].":".$ncount.":".$cnonce.":".$qop_value;
+         $response_value = bin2hex(hmac_md5($A1.":".$string_response.":".$A2));
+         // build the challenge $response
+         $reply = "charset=utf-8,username=\"".$this->user."\",realm=\"".$result['realm']."\",";
+         $reply .= "nonce=\"".$result['nonce']."\",nc=$ncount,cnonce=\"$cnonce\",";
+         $reply .= "digest-uri=\"$digest_uri_value\",response=$response_value,";
+         $reply .= "qop=$qop_value,authzid=\"".utf8_encode($this->auth)."\"";
+         $response = base64_encode($reply);
+         fputs($this->fp, "\"$response\"\r\n");
+     
              $this->line = fgets($this->fp, 1024);
              while(sieve::status($this->line) == F_DATA)
                 $this->line = fgets($this->fp,1024);
@@ -761,24 +789,24 @@ class sieve {
              $this->loggedin = TRUE;
                return TRUE;    
              break;
-	
+    
         case "CRAM-MD5":
-  	     // SASL CRAM-MD5 support works with timsieved 1.1.0
-	     // follows rfc2195 for generating the $response to $challenge
-	     // CRAM-MD5 does not support proxy of $auth by $user
-	     // requires php mhash extension
-	     fputs($this->fp, "AUTHENTICATE \"CRAM-MD5\"\r\n");
-	     // $clen is the length of the challenge line the server gives us
-	     $clen = fgets($this->fp, 1024);
-	     // read for 1024, should be long enough?
-	     $challenge = fgets($this->fp, 1024);
-	     // build a response to the challenge
-	     $hash = bin2hex(hmac_md5(base64_decode($challenge), $this->pass));
-	     $response = base64_encode($this->user." ".$hash);
-	     // respond to the challenge string
-	     fputs($this->fp, "\"$response\"\r\n");
-	     
-             $this->line = fgets($this->fp, 1024);		
+           // SASL CRAM-MD5 support works with timsieved 1.1.0
+         // follows rfc2195 for generating the $response to $challenge
+         // CRAM-MD5 does not support proxy of $auth by $user
+         // requires php mhash extension
+         fputs($this->fp, "AUTHENTICATE \"CRAM-MD5\"\r\n");
+         // $clen is the length of the challenge line the server gives us
+         $clen = fgets($this->fp, 1024);
+         // read for 1024, should be long enough?
+         $challenge = fgets($this->fp, 1024);
+         // build a response to the challenge
+         $hash = bin2hex(hmac_md5(base64_decode($challenge), $this->pass));
+         $response = base64_encode($this->user." ".$hash);
+         // respond to the challenge string
+         fputs($this->fp, "\"$response\"\r\n");
+         
+             $this->line = fgets($this->fp, 1024);        
              while(sieve::status($this->line) == F_DATA)
                 $this->line = fgets($this->fp,1024);
 
@@ -788,26 +816,26 @@ class sieve {
                return TRUE;    
              break;
 
-	case "LOGIN":
- 	     $login=base64_encode($this->user);
- 	     $pass=base64_encode($this->pass);
- 	
- 	     fputs($this->fp, "AUTHENTICATE \"LOGIN\"\r\n");
- 	     fputs($this->fp, "{".strlen($login)."+}\r\n");
- 	     fputs($this->fp, "$login\r\n");
- 	     fputs($this->fp, "{".strlen($pass)."+}\r\n");
- 	     fputs($this->fp, "$pass\r\n");
+    case "LOGIN":
+          $login=base64_encode($this->user);
+          $pass=base64_encode($this->pass);
+     
+          fputs($this->fp, "AUTHENTICATE \"LOGIN\"\r\n");
+          fputs($this->fp, "{".strlen($login)."+}\r\n");
+          fputs($this->fp, "$login\r\n");
+          fputs($this->fp, "{".strlen($pass)."+}\r\n");
+          fputs($this->fp, "$pass\r\n");
  
-	     $this->line=fgets($this->fp,1024);
- 	     while(sieve::status($this->line) == F_HEAD ||
- 	           sieve::status($this->line) == F_DATA)
- 	         $this->line=fgets($this->fp,1024);
- 	
- 	     if(sieve::status($this->line) == F_NO)
- 	         return false;
- 	     $this->loggedin=true;
- 	     return true;
- 	     break;
+         $this->line=fgets($this->fp,1024);
+          while(sieve::status($this->line) == F_HEAD ||
+                sieve::status($this->line) == F_DATA)
+              $this->line=fgets($this->fp,1024);
+     
+          if(sieve::status($this->line) == F_NO)
+              return false;
+          $this->loggedin=true;
+          return true;
+          break;
 
         default:
             return false;
@@ -817,14 +845,11 @@ class sieve {
   }
 
   /**
-   * Return an array of available capabilities.
+   * Read incoming capability response.
    *
    * @return array
    */
-  function sieve_get_capability() {
-    if($this->loggedin==false)
-        return false;
-    fputs($this->fp, "CAPABILITY\r\n"); 
+  function sieve_read_capability_response() {
     $this->line=fgets($this->fp,1024);
 
     $tmp = array();
@@ -862,10 +887,10 @@ class sieve {
               $this->modules = split(' ', $this->item[1]);
               if(is_array($this->modules)){
                   foreach($this->modules as $m) {
-                      $this->capabilities[$cap_type][$m]=true;
+                      $this->capabilities[$cap_type][strtolower($m)]=true;
                   }
               } elseif(is_string($this->modules)) {
-                  $this->capabilites[$cap_type][$this->modules]=true;
+                  $this->capabilites[$cap_type][strtolower($this->modules)]=true;
               }
           } else { 
               $this->capabilities["unknown"][]=$this->line;
@@ -876,6 +901,18 @@ class sieve {
     return $this->capabilities['modules'];
   }
 
+  /**
+   * Return an array of available capabilities.
+   *
+   * @return array
+   */
+  function sieve_get_capability() {
+    if($this->loggedin==false)
+        return false;
+    fputs($this->fp, "CAPABILITY\r\n");
+
+    return $this->sieve_read_capability_response();
+  }
 }
 
 
@@ -949,6 +986,3 @@ function decode_challenge ($input) {
     return $resp;
 }
 
-// vim:ts=4:et:ft=php
-
-?>
